==> kari-0.30.tar.gz: kari-0.30/chloe/file.c <==
/* YAWN, Yet Another Web Navigator */
/* Copyright (C) 1998, James Troup <jjtroup@comp.brad.ac.uk>
 *                     Lee Willis <lwillis@comp.brad.ac.uk> */

/* This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include "neve.h"
#include "smg.h"
#include "utils.h"
#include "file.h"

char *
get_via_file (const BrowserWindow *paula)
{

  FILE *fp;
  char *buffer_p;
  long file_length;
  int n_read;
  char *filename;
  
  filename = xmalloc ( strlen (paula->hostname) + strlen (paula->path) + 2);
  sprintf (filename, "%s%s", paula->hostname, paula->path);

  fp = fopen (filename, "rb");
  if (fp == NULL) {
    error ("get_file: couldn't open file \"%s\"",  filename);
    return NULL;
  }
  
  if ((fseek (fp, 0, SEEK_END)) != 0) {
    error ("get_file: couldn't seek to the end of file \"%s\"", filename);
    return NULL;
  }

  file_length = ftell (fp);
  rewind (fp);

  /* FIXME: this is *not* an error for YAWN; this is a left-over from
   * the {ruari,quinn}-diff origins of this code.  Uh, but HTH do I
   * fix it? */
  
  if (file_length == 0) 
    {
      error ("get_file: \"%s\" is zero-length", filename);
      return NULL;
    }
  
  /* We don't do any memory management, we just ask for a chunk of
   * memory the size of the file */
  buffer_p = xmalloc (file_length+1);
  
  n_read = fread (buffer_p, sizeof (char), file_length, fp);
  if (n_read == 0) 
    {
      error ("get_file: couldn't read file \"%s\" into buffer", filename);
      return NULL;
    }
  else if (n_read < file_length)
    {
      error ("get_file: short read (found %d, expected %ld) in file \"%s\"",
	      n_read, file_length, filename);
    }
  
  if (fclose (fp) == EOF)
    {
      error ("get_file: couldn't close file \"%s\"", filename);
      return NULL;
    }
  
  buffer_p[file_length] = '\0';


  return buffer_p;

}

==> kari-0.32.tar.gz: kari-0.32/chloe/file.c <==
/* YAWN, Yet Another Web Navigator */
/* Copyright (C) 1998, James Troup <jjtroup@comp.brad.ac.uk>
 *                     Lee Willis <lwillis@comp.brad.ac.uk> */

/* This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include "neve.h"
#include "smg.h"
#include "utils.h"
#include "file.h"

char *
get_via_file (const BrowserWindow *paula)
{

  int fd;
  char *filename;
  char *buffer_p;

  filename = xmalloc ( strlen (paula->hostname) + strlen (paula->path) + 2);
  sprintf (filename, "%s%s", paula->hostname, paula->path);

  fd = open (filename, O_RDONLY);
  if (fd < 0) {
    error ("get_file: couldn't open file \"%s\"",  filename);
    return NULL;
  }

  buffer_p = read_from_file (fd, filename);
  free (filename);
  return buffer_p;

}

char *
read_from_file (const int fd, const char *filename)
{

  char *buffer_p;
  long file_length;
  int n_read;
  struct stat buf;
  
  if ((fstat (fd, &buf)) != 0)
    {
      error ("get_file: fstat on %s failed", filename);
      return NULL;
    }
  
  file_length = buf.st_size;
  /* FIXME: this is *not* an error for YAWN; this is a left-over from
   * the {ruari,quinn}-diff origins of this code.  Uh, but HTH do I
   * fix it? */
  
  if (file_length == 0) 
    {
      error ("get_file: \"%s\" is zero-length", filename);
      return NULL;
    }
  
  /* We don't do any memory management, we just ask for a chunk of
   * memory the size of the file */
  buffer_p = xmalloc (file_length+1);
  
  n_read = read (fd, buffer_p, file_length);
  if (n_read < 0) 
    {
      error ("get_file: couldn't read file \"%s\" into buffer", filename);
      return NULL;
    }
  else if (n_read < file_length)
    {
      error ("get_file: short read (found %d, expected %ld) in file \"%s\"",
	      n_read, file_length, filename);
    }
  
  if (close (fd))
    {
      error ("get_file: couldn't close file \"%s\"", filename);
      return NULL;
    }
  
  buffer_p[file_length] = '\0';

  return buffer_p;

}
